@@ -40,33 +40,35 @@ static const struct
   pa_channel_position_t pa_pos;
 } gst_pa_pos_table[] = {
   {
-  GST_AUDIO_CHANNEL_POSITION_MONO, PA_CHANNEL_POSITION_MONO}, {
-  GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT, PA_CHANNEL_POSITION_FRONT_LEFT}, {
-  GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT, PA_CHANNEL_POSITION_FRONT_RIGHT}, {
-  GST_AUDIO_CHANNEL_POSITION_REAR_CENTER, PA_CHANNEL_POSITION_REAR_CENTER}, {
-  GST_AUDIO_CHANNEL_POSITION_REAR_LEFT, PA_CHANNEL_POSITION_REAR_LEFT}, {
-  GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT, PA_CHANNEL_POSITION_REAR_RIGHT}, {
-  GST_AUDIO_CHANNEL_POSITION_LFE1, PA_CHANNEL_POSITION_LFE}, {
-  GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER, PA_CHANNEL_POSITION_FRONT_CENTER}, {
-  GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER,
-        PA_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER}, {
-  GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER,
-        PA_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER}, {
-  GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT, PA_CHANNEL_POSITION_SIDE_LEFT}, {
-  GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT, PA_CHANNEL_POSITION_SIDE_RIGHT}, {
-  GST_AUDIO_CHANNEL_POSITION_TOP_CENTER, PA_CHANNEL_POSITION_TOP_CENTER}, {
-  GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_LEFT,
-        PA_CHANNEL_POSITION_TOP_FRONT_LEFT}, {
-  GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_RIGHT,
-        PA_CHANNEL_POSITION_TOP_FRONT_RIGHT}, {
-  GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_CENTER,
-        PA_CHANNEL_POSITION_TOP_FRONT_CENTER}, {
-  GST_AUDIO_CHANNEL_POSITION_TOP_REAR_LEFT, PA_CHANNEL_POSITION_TOP_REAR_LEFT}, {
-  GST_AUDIO_CHANNEL_POSITION_TOP_REAR_RIGHT,
-        PA_CHANNEL_POSITION_TOP_REAR_RIGHT}, {
-  GST_AUDIO_CHANNEL_POSITION_TOP_REAR_CENTER,
-        PA_CHANNEL_POSITION_TOP_REAR_CENTER}, {
-  GST_AUDIO_CHANNEL_POSITION_NONE, PA_CHANNEL_POSITION_INVALID}
+      GST_AUDIO_CHANNEL_POSITION_MONO, PA_CHANNEL_POSITION_MONO}, {
+      GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT, PA_CHANNEL_POSITION_FRONT_LEFT}, {
+      GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT, PA_CHANNEL_POSITION_FRONT_RIGHT}, {
+      GST_AUDIO_CHANNEL_POSITION_REAR_CENTER, PA_CHANNEL_POSITION_REAR_CENTER}, {
+      GST_AUDIO_CHANNEL_POSITION_REAR_LEFT, PA_CHANNEL_POSITION_REAR_LEFT}, {
+      GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT, PA_CHANNEL_POSITION_REAR_RIGHT}, {
+      GST_AUDIO_CHANNEL_POSITION_LFE1, PA_CHANNEL_POSITION_LFE}, {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+      PA_CHANNEL_POSITION_FRONT_CENTER}, {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER,
+      PA_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER}, {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER,
+      PA_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER}, {
+      GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT, PA_CHANNEL_POSITION_SIDE_LEFT}, {
+      GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT, PA_CHANNEL_POSITION_SIDE_RIGHT}, {
+      GST_AUDIO_CHANNEL_POSITION_TOP_CENTER, PA_CHANNEL_POSITION_TOP_CENTER}, {
+        GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_LEFT,
+      PA_CHANNEL_POSITION_TOP_FRONT_LEFT}, {
+        GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_RIGHT,
+      PA_CHANNEL_POSITION_TOP_FRONT_RIGHT}, {
+        GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_CENTER,
+      PA_CHANNEL_POSITION_TOP_FRONT_CENTER}, {
+        GST_AUDIO_CHANNEL_POSITION_TOP_REAR_LEFT,
+      PA_CHANNEL_POSITION_TOP_REAR_LEFT}, {
+        GST_AUDIO_CHANNEL_POSITION_TOP_REAR_RIGHT,
+      PA_CHANNEL_POSITION_TOP_REAR_RIGHT}, {
+        GST_AUDIO_CHANNEL_POSITION_TOP_REAR_CENTER,
+      PA_CHANNEL_POSITION_TOP_REAR_CENTER}, {
+      GST_AUDIO_CHANNEL_POSITION_NONE, PA_CHANNEL_POSITION_INVALID}
 };
 
 static gboolean
@@ -450,7 +452,7 @@ gst_pulse_format_info_to_caps (pa_format_info * format)
       if (pa_format_info_get_prop_string (format,
               PA_PROP_FORMAT_SAMPLE_FORMAT, &tmp)) {
         /* No specific sample format means any sample format */
-        ret = gst_caps_from_string (_PULSE_CAPS_PCM);
+        ret = gst_pulse_fix_pcm_caps (gst_caps_from_string (_PULSE_CAPS_PCM));
         goto out;
 
       } else if (ss.format == PA_SAMPLE_ALAW) {
@@ -507,3 +509,46 @@ gst_pulse_format_info_to_caps (pa_format_info * format)
 out:
   return ret;
 }
+
+GstCaps *
+gst_pulse_fix_pcm_caps (GstCaps * incaps)
+{
+  GstCaps *outcaps;
+  int i;
+
+  outcaps = gst_caps_make_writable (incaps);
+
+  for (i = 0; i < gst_caps_get_size (outcaps); i++) {
+    GstStructure *st = gst_caps_get_structure (outcaps, i);
+    const gchar *format = gst_structure_get_name (st);
+    const GValue *value;
+    GValue new_value = G_VALUE_INIT;
+    gint min, max, step;
+
+    if (!(g_str_equal (format, "audio/x-raw") ||
+            g_str_equal (format, "audio/x-alaw") ||
+            g_str_equal (format, "audio/x-mulaw")))
+      continue;
+
+    value = gst_structure_get_value (st, "rate");
+
+    if (!GST_VALUE_HOLDS_INT_RANGE (value))
+      continue;
+
+    min = gst_value_get_int_range_min (value);
+    max = gst_value_get_int_range_max (value);
+    step = gst_value_get_int_range_step (value);
+
+    if (min > PA_RATE_MAX)
+      min = PA_RATE_MAX;
+    if (max > PA_RATE_MAX)
+      max = PA_RATE_MAX;
+
+    g_value_init (&new_value, GST_TYPE_INT_RANGE);
+    gst_value_set_int_range_step (&new_value, min, max, step);
+
+    gst_structure_take_value (st, "rate", &new_value);
+  }
+
+  return outcaps;
+}
